@@ -5,9 +5,6 @@ from pyg_base._dict import Dict
 from pyg_base._cache import cache
 import datetime
 from dateutil.rrule import rrule, MONTHLY, WEEKLY, DAILY, YEARLY, HOURLY, MINUTELY, SECONDLY, MO, TU, WE, TH, FR, SA, SU
-from dateutil import tz
-from pytz import country_timezones
-
 
 import numpy as np
 import re
@@ -23,22 +20,6 @@ isoformat = re.compile('^[0-9]{4}-[0-9]{2}-[0-9]{2}T')
     
 __all__ = ['date_range', 'drange', 'calendar', 'Calendar', 'clock', 'as_time']
 
-@cache
-def tzones():
-    x = {iso :country_timezones[iso] for iso in country_timezones}
-    tzs = {iso: code[0] for iso, code in x.items() if len(code) == 1}
-    for iso, code in x.items():
-        tzs.update({c.split('/')[-1].replace('_', ' ').lower(): c for c in code})
-    tzs  = {key : tz.gettz(value) for key, value in tzs.items()}
-    t = datetime.datetime.now()
-    tzs.update({v.tzname(t): v for v in tzs.values() if v is not None})
-    tzs['WET'] = tzs['western european'] = tzs['MA'] # Malta
-    tzs['EET'] = tzs['eastern european'] = tzs['GR']
-    tzs['CET'] = tzs['cental european'] = tzs['AT'] # austria
-    tzs['EST'] = tzs['eastern standard'] = tzs['new york']
-    tzs['CST'] = tzs['central standard'] = tzs['chicago']
-    tzs['greenwich mean time'] = tzs['GMT']
-    return tzs
 
 def as_time(t = None):
     """
@@ -630,6 +611,19 @@ def calendar(key = None, holidays = None, weekend = None, t0 = None, t1 = None):
         calendars[key] = Calendar(key, holidays = holidays, weekend = weekend, t0 = t0, t1 = t1)    
     return calendars[key]
 
+
+@np.vectorize
+def _weekday_intraday_clock(t):
+    t = dt(t)
+    j = ymd(t)
+    c = calendar()._populate()
+    if j in c.dt2int:
+        i = t - j
+        return c.dt2int[j] + (i.microseconds / 1e6 + i.seconds) / 86400
+    else:
+        j = c.adjust(j)        
+        return c.dt2int[j]
+
 @np.vectorize
 def _fraction_clock(t):
     t = dt(t)
@@ -669,7 +663,8 @@ _clocks = dict(f = _fraction_clock,
                w = _week_clock,
                m = _month_clock,
                y = _year_clock,
-               q = _quarter_clock
+               q = _quarter_clock,
+               k = _weekday_intraday_clock
                )
 
 
@@ -730,11 +725,15 @@ def clock(ts, time = None, t = None):
     elif is_ts(ts):    
         if isinstance(time, Calendar):
             return np.vectorize(time.clock)(ts.index)
+        elif callable(time):
+            return np.vectorize(time)(ts.index)            
         elif time == 'b':
             time = calendar()
             return np.vectorize(time.clock)(ts.index)
         elif time in _clocks:
             return _clocks[time](ts.index)
+        else:
+            raise ValueError('cannot determine time for ts=%s, time=%s'%(ts, time))            
     else:
         raise ValueError('cannot determine time for ts=%s, time=%s'%(ts, time))
 
